@@ -6,6 +6,8 @@ use serde::{Deserialize, Serialize};
 use serde_json::{Result, Value};
 use std::collections::HashMap;
 
+use super::message::WebsocketMsg;
+
 const EXCHANGE_NAME: &str = "huobi";
 
 // see https://huobiapi.github.io/docs/spot/v1/en/#trade-detail
@@ -40,13 +42,6 @@ struct TradeTick {
     data: Vec<SpotTradeMsg>,
 }
 
-#[derive(Serialize, Deserialize)]
-struct WebsocketMsg<T: Sized> {
-    ch: String,
-    ts: i64,
-    tick: T,
-}
-
 pub(super) fn parse_trade(msg: &str) -> Result<Vec<TradeMsg>> {
     let ws_msg = serde_json::from_str::<WebsocketMsg<TradeTick>>(msg)?;
 
